use strict;
use warnings;
use Getopt::Long;
use Time::HiRes qw( time );

use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::GoogleBase::API::Cartridge;
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );


my $Output;
#========================================================================================
# $function         InitializeEoLGoogleShopping
# $state            public
#----------------------------------------------------------------------------------------
# $syntax           InitializeEoLGoogleShopping();
#----------------------------------------------------------------------------------------
# $description      Adds and activates the notification for the EoL Google Shopping.
#                   notification for all unity shops with enabled Google Shopping.
#========================================================================================
sub InitializeEoLGoogleShopping {
    print "Reading all Unity shops\n";
    my $aShopIDs = GetUnityShopIDs();

    my $Cartridge = DE_EPAGES::GoogleBase::API::Cartridge->new;

    foreach my $ShopID ( @$aShopIDs ) {
        my $Shop = LoadObject($ShopID);

        # import shop notifications
        if (defined $Shop->get('PortalConfigs')) {
            foreach my $PortalConfig (@{$Shop->get('PortalConfigs')}){
                next unless ($PortalConfig->class()->isSubClassOf('GoogleBaseConfig'));
                eval {
                ImportObjects($Cartridge->xmlDirectory() . '/ShopNotificationEoLGoogleShopping.xml', $Shop->pathString)
                    unless $Shop->folder('ShopNotifications')->existsChild('EoLGoogleShopping');
                };
                warn('Patch failed in shop ' . $ShopID . " with error \"" . GetError() . "\".") if ExistsError();
                print "Notification imported. ShopID: ". $ShopID.", Shop path:  ".$Shop->pathString." \n" if $Output;
                last;
            }
        }
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'output!'     => \$Output,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { InitializeEoLGoogleShopping(); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Adds and activates the notification about the Google shopping EoL to
    all existing Unity shops with working google shopping.
Usage:
    perl $0 [options] [flags] datafiles
options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options
    -output     output for each shop the script runs
Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript('Sub' => \&Main);
