#==============================================================================
# package      DE_EPAGES::MolliePayments::Hooks::OrderProcess
# state        private
#==============================================================================
package DE_EPAGES::MolliePayments::Hooks::OrderProcess;

use strict;
use DE_EPAGES::MolliePayments::API::Requests::Methods;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::MolliePayments::API::MolliePaymentsClient;
use DE_EPAGES::MolliePayments::UI::BasketOrder;
use DE_EPAGES::MolliePayments::API::PaymentStatusService;
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::MolliePayments::API::Constants qw (
    PAYMENT_TYPE_MOLLIE_PAYMENTS
    LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS
    PAYMENT_STATUS_PAID
);

#========================================================================================
# function     OnBasket2Order
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2Order($hParams);
#----------------------------------------------------------------------------------------
# description  API_Basket2Order triggered, set payment amount
#               Triggered for classic/individual steps, but at the moment restricted to Unity shops, which means individual only
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments | ref.hash.*
#========================================================================================
sub OnBasket2Order {
    my ($hParams) = @_;

    my $Basket = $hParams->{'Basket'};

    my $LineItemPayment = $Basket->container()->get('Payment');
    return unless defined($LineItemPayment);

    my $PaymentMethod = $LineItemPayment->get('PaymentMethod');
    return unless defined($PaymentMethod);

    if ($PaymentMethod->instanceOf('PaymentMethodMolliePayments')) {
        $LineItemPayment->updateOnExecute($hParams);
    }

    return;
}

sub OnUI_ViewMultiCheckoutPayment {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $Shop = $Servlet->object->getSite;

    my $Payments = $Shop->get('PaymentMethods');

    my $PaymentMethod;
    foreach my $Method ( @{$Payments} ) {
        if ($Method->instanceOf('PaymentMethodMolliePayments') && $Method->get('Method') eq 'ideal') {
            $PaymentMethod = $Method;
        }
    }

    my $ResponseIssuers = _handleIdeal($Servlet, $PaymentMethod);

    return $ResponseIssuers;
}
#========================================================================================
# function     BeforeSaveMultiCheckoutConfirmation
# state        public
#----------------------------------------------------------------------------------------
# syntax       BeforeSaveMultiCheckoutConfirmation($hParams);
#----------------------------------------------------------------------------------------
# description  check order payment status and redirect to error or success
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub BeforeSaveMultiCheckoutConfirmation {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object();

    my $LineItemContainer = $Basket->container();
    return unless ( defined($LineItemContainer));

    my $LineItemPayment = $LineItemContainer->get('Payment');
    return unless ( defined($LineItemPayment));
    return unless ( $LineItemPayment->instanceOf(LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS));

    my $hFormValues = $Servlet->form()->values();
    my $aChangeActions = $hFormValues->{'ChangeAction'};

    return unless ( 'SaveMultiCheckoutConfirmation' ~~ $aChangeActions );

    DE_EPAGES::MolliePayments::UI::BasketOrder->ValidateOrderPaymentStatus($Servlet, $LineItemPayment);

    return;
}

#========================================================================================
# function     OrderCreatedMultiStepCheckout
# state        public
#----------------------------------------------------------------------------------------
# syntax       OrderCreatedMultiStepCheckout($hParams);
#----------------------------------------------------------------------------------------
# description  Set Order to paid
#               Attention: this subroutine is used for both classic/individual variants of the 'OrderCreated' hook
#               The payloads are the same
#               !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#               UI_OrderCreatedMultiStepCheckout
#               UI_OrderCreated
#               !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Order - order - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OrderCreatedMultiStepCheckout {
    my ($hParams) = @_;

    my $Order = $hParams->{'Order'};

    my $LineItemContainer = $Order->container();
    return unless ( defined($LineItemContainer));

    my $LineItemPayment = $LineItemContainer->get('Payment');
    return unless ( defined($LineItemPayment));
    return unless ( $LineItemPayment->instanceOf(LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS));

    if ($LineItemPayment->get('TransStatus') eq PAYMENT_STATUS_PAID) {
        LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, 'In OrderCreatedMultiStepCheckout set Order PaidOn', {'OrderId' => $Order->id()});
        DE_EPAGES::MolliePayments::API::PaymentStatusService->setOrderStatusPaidTime($Order, $LineItemPayment->get('TransTime'));
    }

    return;
}

sub OnUI_ViewMultiCheckoutConfirmation {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};

    $Servlet->vars('idealIssuer', $Servlet->form->value('idealIssuers'));

    return;
}

sub _handleIdeal {
    my ($Servlet, $Payment) = @_;

    return unless defined $Payment;

    my $IsInTestMode = $Payment->get('IsInTestMode');
    my $ProfileId = $Payment->get('ProfileId');

    my $hMethodsData = {
        'payment' => $Payment,
    };
    my $MethodsHandler = DE_EPAGES::MolliePayments::API::Requests::Methods->new($hMethodsData);
    $MethodsHandler->withProfileId($ProfileId);
    # set the path to change the handling of endpoint building in mollie client
    $MethodsHandler->path('ideal');
    my $MollieService = DE_EPAGES::MolliePayments::API::MolliePaymentsClient->new($MethodsHandler);

    my $hLogParams = {
        'PaymentMethodId' => $Payment->id(),
        'profileId' => $ProfileId,
    };
    my $hMethodsResponse = {};

    eval {
        $hMethodsResponse = $MollieService->withTestMode($IsInTestMode)->doGetRequest();
    };
    if (ExistsError()) {
        my $Error = GetError();

        $hLogParams->{'Error'} = $Error->message();
        _logPayment('Error when getting Ideal Issuers', $hLogParams, undef, undef);
        $Servlet->vars('MollieConnectionError', 1);
        return;
    }

    $hLogParams->{'MethodsResponse'} = $hMethodsResponse;
    _logPayment('Get Ideal Issuers response', $hLogParams, undef , undef);
    $Servlet->vars('IdealIssuers', $hMethodsResponse->{'issuers'});

    return $hMethodsResponse->{'issuers'};
}

sub _logPayment {
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, $Name, $DebugData, $Section, $Filter);
}

1;
