#==============================================================================
# package      DE_EPAGES::PaymentTypes::Hooks::OrderProcess
# state        private
#==============================================================================
package DE_EPAGES::PaymentTypes::Hooks::OrderProcess;

use strict;

use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( SubtractDuration GetCurrentDateTime );
use DE_EPAGES::Core::API::TestParameter qw (TestIBAN TestBIC);


#========================================================================================
# function     OnViewMultiCheckoutPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnViewMultiCheckoutPayment($hParams);
#----------------------------------------------------------------------------------------
# description  handling for direct debit
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnViewMultiCheckoutPayment {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    # test if ve have a correct iban on direct debit payment otherwise return to payment
    my $Basket = $Servlet->object;

    # test if we have the correct payment instance and currency
    return unless _testPayment($Basket);

    # get bank account data and test IBAN
    my $User = $Basket->get('User');
    my $BillingAddress = $User->get('BillingAddress');

    # test
    my $IBANIsValid = _TestIBAN($BillingAddress);
    $Servlet->vars('IBAN_VALID',$IBANIsValid);

    # test
    my $BICIsValid = _TestBIC($BillingAddress);
    $Servlet->vars('BIC_VALID',$BICIsValid);

}

#========================================================================================
# function     OnSaveMultiCheckoutPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnSaveMultiCheckoutPayment($hParams);
#----------------------------------------------------------------------------------------
# description  handling for direct debit
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnSaveMultiCheckoutPayment {
    my ($hParams) = @_;

}

sub _testPayment {
    my ($Basket) = @_;

    my $LineItemContainer = $Basket->container;
    # test if we have a direct debit payment
    return undef unless defined $LineItemContainer;
    return undef unless $LineItemContainer->get('CurrencyID') eq 'EUR';
    my $PaymentLineItem = $LineItemContainer->get('Payment');
    return undef unless defined $PaymentLineItem;
    return undef unless $PaymentLineItem->instanceOf('LineItemPaymentDirectDebit');

    return 1;
}


sub _TestIBAN {
    my ($BillingAddress) = @_;

    my $BankAccountNo = $BillingAddress->get('BankAccountNo');
    return undef unless length($BankAccountNo);

    eval {
        TestIBAN('IBAN',$BankAccountNo,1);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'FORMAT_NOT_IBAN') {
            return 0;
        }
    }
    return 1;
}

sub _TestBIC {
    my ($BillingAddress) = @_;

    my $BankCode = $BillingAddress->get('BankCode');
    return undef unless length($BankCode);

    eval {
        TestBIC('BIC',$BankCode);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'FORMAT_NOT_BIC') {
            return 0;
        }
    }
    return 1;
}

#========================================================================================
# function     OnOrderCreated
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnOrderCreated($hParams);
#----------------------------------------------------------------------------------------
# description  API_OrderCreated triggered, set parameters to customer and order,
#               if DirectDebit was used
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments | ref.hash.*
#========================================================================================
sub OnOrderCreated {
    my ($hParams) = @_;
    my $Order = $hParams->{'Order'};

    # do we have a customer ?
    my $Customer = $Order->get('Customer') // return;

    # test if we have a direct debit payment
    my $LineItemContainer = $Order->container;
    return undef unless defined $LineItemContainer;
    my $PaymentLineItem = $LineItemContainer->get('Payment');
    return undef unless defined $PaymentLineItem;
    return undef unless $PaymentLineItem->instanceOf('LineItemPaymentDirectDebit');

    # do we use the EUR as lineitencontainer currency
    return unless $LineItemContainer->get('CurrencyID') eq 'EUR';


    # if we use direct debit the order item LineItemPaymentDirectDebit do the following:
    my $IsMandRefValid = 1;

    # test if the customer attribute IsMandateIDValid is set by changing the
    # bank account data on the address
    $IsMandRefValid = 0 unless $Customer->get('IsMandateIDValid');

    # do we have a reference number issue date?
    my $Now = GetCurrentDateTime();
    my $LastSEPAOrderDate = $Customer->get('MandateIDLastOrderDate');
    $IsMandRefValid = 0 unless $LastSEPAOrderDate;

    if (not defined  $Customer->get('MandateID')) {
        # we do not have a mandate reference nuber.
        $IsMandRefValid = 0;
    }

    if ($IsMandRefValid) {
        # test mandate reference number, if last direct debit order with this number is not older than 3 yrs.
        my $BaseDateTime = SubtractDuration($Now->clone, 'years', 3);
        $IsMandRefValid = 0 if $BaseDateTime > $LastSEPAOrderDate ;
    }

    #-- UNITY-8436
    #-- Regenrate the Mandate ID for EVERY order
    $IsMandRefValid = 0;

    # create a new mandate reference number.
    my $hCParams;
    if ($IsMandRefValid == 0) {
        # create a new mandate reference number
        my $NewMandateID = $Customer->alias . '-' . $Now->dmy('') .'-'. $Now->hms('') ;
        $hCParams->{'MandateID'} = $NewMandateID;
        $hCParams->{'IsMandateIDValid'} = 1;
    }

    # set the NewMandateIDLastOrderDate to the customer, if number is new
    # set the last order date with this number
    $hCParams->{'MandateIDLastOrderDate'} = $Now;
    $Customer->set($hCParams);


    # set the attribute: MandateID and CreditorID to the to the order payment lineitem
    my $hPParams;
    $hPParams->{'MandateID'} = $Customer->get('MandateID');
    $hPParams->{'CreditorID'} = $PaymentLineItem->get('PaymentMethod')->get('CreditorID');
    $PaymentLineItem->set($hPParams);

    return;

}

#========================================================================================
# function     OnBeforeEditOrder2Order
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeEditOrder2Order($hParams);
#----------------------------------------------------------------------------------------
# description  BeforeEditOrder2Order triggered, set parameters to customer and order,
#               if DirectDebit was used. function OnOrderCreated() is used
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments | ref.hash.*
#========================================================================================
sub OnBeforeEditOrder2Order {
    my ($hParams) = @_;
    return OnOrderCreated($hParams);

}

1;


