#========================================================================================
# package      DE_EPAGES::Saferpay2::UI::Shop
# base         DE_EPAGES::Order::UI::Shop
# state        public
#----------------------------------------------------------------------------------------
# description  UI Controller for shop
#========================================================================================
package DE_EPAGES::Saferpay2::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByGUID LoadObjectByGUID Transaction );
use DE_EPAGES::Order::UI::Basket;

use DE_EPAGES::Saferpay2::API::Constants qw (
    INTERNAL_CARTRIDGE_NAME
    PAYMENT_TYPE_SAFERPAY2
    AUTHORIZED_PAYMENT_STATUS
    CAPTURED_PAYMENT_STATUS
    FAILED_PAYMENT_STATUS
    FAILED_REPEAT_PAYMENT_STATUS
);
use DE_EPAGES::Saferpay2::API::SaferpayService;
use DE_EPAGES::Saferpay2::API::Tools qw ( HandleTransactionResponse );


use strict;

use constant ALREADY_CAPTURED_ERROR_MESSAGE => 'TRANSACTION_ALREADY_CAPTURED';

#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#----------------------------------------------------------------------------------------
# description  Name of PaymentType for search/view transactions.
#----------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName {
    return INTERNAL_CARTRIDGE_NAME;
}


#========================================================================================
# function     PaymentBacklinkSaferpay2
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentBacklinkSaferpay2( $Servlet );
#----------------------------------------------------------------------------------------
# description  Redirect URL for the shopper in case of success or error
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub PaymentBacklinkSaferpay2 {
    my $self = shift;
    my ($Servlet) = @_;

    my $hServletValues = $self->_retrieveServletValues($Servlet);

    my $hBackCall = $self->_processBackCall($Servlet, 'PaymentBacklinkSaferpay2');

    if ( $hBackCall->{'ValidCall'} ) {
        my $Shop = $hServletValues->{'Shop'};
        my $LineItemPayment = $hBackCall->{'LineItemPayment'};
        # Get BasketOrder from LineItemPayment because '_processBackCall' may have been created an order.
        my $BasketOrder = $self->_getBasketOrderFromLineItem($LineItemPayment);

        $Servlet->vars('ViewObjectID', $BasketOrder->id());

        if ($hBackCall->{'Transaction'}->{'Status'} eq FAILED_REPEAT_PAYMENT_STATUS) {
            $Servlet->vars('SaferpayPaymentFailed', 1);
            $Servlet->vars('SaferpayPaymentFailedReason', $hBackCall->{'Transaction'}->{'Error'});

            $Servlet->vars('ViewAction', 'View'); # Classic and Mobile checkouts
            if ($Shop->get('OrderProcess') and not $Servlet->vars('IsMobile')) { # MultiStep checkout
                $Servlet->vars('ViewAction', 'ViewMultiCheckoutPayment');
            }
        } else {
            if ($Shop->isUnity()) {
                $Servlet->vars('ViewAction', 'ViewMultiCheckoutCustomerOrder');
            }
            if ($hBackCall->{'Transaction'}->{'Status'} eq FAILED_PAYMENT_STATUS) {
                $Servlet->vars('SaferpayPaymentFailed', 1);
                $Servlet->vars('SaferpayPaymentFailedReason', $hBackCall->{'Transaction'}->{'Error'});

                if ($hServletValues->{'Shop'}->isUnity()) {
                    $LineItemPayment->set({'PaymentErrorCode' => 'Failed'});
                }
            }
        }
    }

    return;
}


#========================================================================================
# function     PaymentCallbackSaferpay2
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentCallbackSaferpay2( $Servlet );
#----------------------------------------------------------------------------------------
# description  Server-server callback from payment provider. Webhook for status change to
#               success
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub PaymentCallbackSaferpay2 {
    my $self = shift;
    my ($Servlet) = @_;

    my $hServletValues = $self->_retrieveServletValues($Servlet);

    eval {
        $self->_processBackCall($Servlet, 'PaymentCallbackSaferpay2');
    };
    if (ExistsError()) {
        my $Error = GetError();
        $self->_LogPayment('PaymentCallbackSaferpay2 failed', {
            'Error'               => $Error->message(),
            'OrderId'             => $hServletValues->{'BasketOrderId'},
            'LineItemPaymentGUID' => $hServletValues->{'LineItemPaymentGUID'}
        });
    }

    $Servlet->params('ContentType', 'text/plain');
    $Servlet->params('ExitAfterEvent', 1);
    $Servlet->writeContentOutput('OK');
    return;
}


sub CaptureTransactions {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my @Transactions = $Form->value('ListObjectID');

    my $Shop = $Servlet->object();

    foreach my $TransactionID (@Transactions) {
        my $LineItemPayment = $Shop->loadObject($TransactionID, 'LineItemPayment');
        my $TransStatus = $LineItemPayment->get('TransStatus');
        my $TransNumber = $LineItemPayment->get('TransID');

        $self->_LogPayment('CaptureTransactions', {
            'transID' => $TransactionID,
            'transStatus' => $TransStatus,
            'transNumber' => $TransNumber,
        });
        #Only capture authorized Transactions
        if ($TransStatus eq AUTHORIZED_PAYMENT_STATUS and $TransNumber) {
            $self->_LogPayment('CaptureTransactions', {$TransactionID => AUTHORIZED_PAYMENT_STATUS});
            my $SaferpayService = DE_EPAGES::Saferpay2::API::SaferpayService->new($LineItemPayment);
            my $hResponse;
            eval {
                $hResponse = $SaferpayService->CaptureTransaction($LineItemPayment->get('TransID'));
            };
            if (ExistsError() || defined($hResponse->{'ErrorName'})) {
                $self->_handleCaptureError($hResponse, $LineItemPayment, $Form, $TransNumber);
            }
            if ($hResponse->{'Status'} eq CAPTURED_PAYMENT_STATUS) {
                $LineItemPayment->updateTransactionStatus($TransNumber, $hResponse->{'Status'}, $hResponse->{'Status'});
            }
        } else {
            $self->_handleCaptureStatusesExceptions($TransStatus, $Form, $TransNumber);
        }
    }
    $Form->executeFormError;
    return;
}

sub _instantCaptureTransaction {
    my $self = shift;
    my ($SaferpayService, $LineItemPayment, $TransNumber) = @_;

    my $hResponse;
    eval {
        $hResponse = $SaferpayService->CaptureTransaction($TransNumber);
    };
    if (ExistsError() || defined($hResponse->{'ErrorName'})) {
        my $ErrorMessage = GetError() // $hResponse->{'ErrorName'};
        $self->_LogPayment('Instant Capture Transaction', {'ERROR' => $ErrorMessage});
        if ($ErrorMessage eq ALREADY_CAPTURED_ERROR_MESSAGE) {
            $LineItemPayment->updateTransactionStatus($TransNumber, CAPTURED_PAYMENT_STATUS, CAPTURED_PAYMENT_STATUS);
        }
    }
    if ($hResponse->{'Status'} eq CAPTURED_PAYMENT_STATUS) {
        $LineItemPayment->updateTransactionStatus($TransNumber, $hResponse->{'Status'}, $hResponse->{'Status'});
    }
}

sub _handleCaptureStatusesExceptions {
    my $self = shift;
    my ($TransStatus, $Form, $TransNumber) = @_;

    if ($TransStatus eq CAPTURED_PAYMENT_STATUS) {
        $self->_addFormError($Form, 'ALREADY_CAPTURED', $TransNumber);
        return;
    }
    if ($TransNumber ne 'NONE') {
        $self->_addFormError($Form, 'WRONG_STATUS', $TransNumber);
        return;
    }
    $self->_addFormError($Form, 'INVALID_TRANSACTION_ID', $TransNumber);
    return;
}

sub _handleCaptureError {
    my $self = shift;
    my ($hResponse, $LineItemPayment, $Form, $TransNumber) = @_;

    my $ErrorMessage = GetError() // $hResponse->{'ErrorName'};
    $self->_LogPayment('CaptureTransactions', {'ERROR' => $ErrorMessage});

    if ($hResponse->{'ErrorName'} eq 'TRANSACTION_ALREADY_CAPTURED') {
        $LineItemPayment->updateTransactionStatus($TransNumber, CAPTURED_PAYMENT_STATUS, CAPTURED_PAYMENT_STATUS);
        $self->_addFormError($Form, 'ALREADY_CAPTURED', $TransNumber);
        return;
    }
    $self->_addFormError($Form, 'CAPTURE_FAILED', $TransNumber);
    return;
}


sub _updateTransactionStatus {
    my $self = shift;
    my ($SaferpayService, $LineItemPayment, $hResponse) = @_;

    $self->_updateLiabilityShift($LineItemPayment, $hResponse);
    $LineItemPayment->updateTransactionStatus(
        $hResponse->{'Transaction'}->{'Id'},
        $hResponse->{'Transaction'}->{'Status'},
        $hResponse->{'Transaction'}->{'ExternalStatus'}
    );

    my $PaymentMethod = $LineItemPayment->get('PaymentMethod');
    if ($PaymentMethod->get('IsDirectCapture') && ($hResponse->{'Transaction'}->{'Status'} eq AUTHORIZED_PAYMENT_STATUS)) {
        $self->_instantCaptureTransaction($SaferpayService, $LineItemPayment, $hResponse->{'Transaction'}->{'Id'});
    }
}


sub _updateLiabilityShift {
    my $self = shift;
    my ($LineItemPayment, $hResponse) = @_;

    if (isValid($hResponse->{'Liability'}) && isValid($hResponse->{'Liability'}->{'LiabilityShift'})) {
        $LineItemPayment->updateLiabilityShift($hResponse->{'Liability'}->{'LiabilityShift'});
    }
}

sub _addFormError {
    my $self = shift;
    my ($Form, $Reason, $TransactionId) = @_;

    $Form->addFormError({
        Reason => $Reason,
        TransactionId => $TransactionId,
        LoopName => 'ListObjectIDs',
        Form => 'Transactions',
        Name => 'ListObjectID'
    });
}

sub _processBackCall {
    my $self = shift;
    my ($Servlet, $CallType) = @_;

    my $hServletValues = $self->_retrieveServletValues($Servlet);

    $self->_LogPayment($CallType, {
        'BasketOrderId' => $hServletValues->{'BasketOrderId'},
        'LineItemPaymentGUID' => $hServletValues->{'LineItemPaymentGUID'},
    });

    Error('LineItemPaymentGUID_NOT_FOUND') unless (ExistsObjectByGUID($hServletValues->{'LineItemPaymentGUID'}));
    my $LineItemPayment = LoadObjectByGUID($hServletValues->{'LineItemPaymentGUID'});

    if (!$self->_matchLineItemPaymentInShop($hServletValues->{'Shop'}, $LineItemPayment)) {
        if ($CallType eq 'PaymentCallbackSaferpay2') {
            Error('LineItemPaymentGUID_Shop_MISMATCH');
        } else {
            return {
                'ValidCall'       => 0,
                'Transaction'     => {},
                'LineItemPayment' => undef
            }
        }
    }

    my $SaferpayService = DE_EPAGES::Saferpay2::API::SaferpayService->new($LineItemPayment);
    my $Token = $LineItemPayment->get('TransID2');

    my $hResponse = $SaferpayService->DoAssertCall($Token, $CallType);
    $hResponse = HandleTransactionResponse($hResponse);

    # Transactional step for Callback and Backlink calls.
    Transaction(
        sub {
            $self->_updatePaymentStatusAndOrder(
                $Servlet, $SaferpayService, $LineItemPayment, $hResponse
            );
        }
    );

    return {
        'ValidCall'       => 1,
        'Transaction'     => {
            'Status' => $hResponse->{'Transaction'}->{'Status'},
            'Error'  => $hResponse->{'ErrorName'}
        },
        'LineItemPayment' => $LineItemPayment
    };
}

sub _updatePaymentStatusAndOrder {
    my $self = shift;
    my ($Servlet, $SaferpayService, $LineItemPayment, $hResponse) = @_;

    my $BasketOrder = $self->_getBasketOrderFromLineItem($LineItemPayment);

    if ($BasketOrder->instanceOf('CustomerOrder')) {
        $self->_updateTransactionStatus($SaferpayService, $LineItemPayment, $hResponse);
    }
    else {
        if ($hResponse->{'HasErrorsInResponse'}) {
            $LineItemPayment->set({ 'TransStatus' => FAILED_REPEAT_PAYMENT_STATUS });
            $hResponse->{'Transaction'}->{'Status'} = FAILED_REPEAT_PAYMENT_STATUS;
        }
        else {
            DE_EPAGES::Order::UI::Basket->doBasket2Order($Servlet, $BasketOrder);
            $self->_updateTransactionStatus($SaferpayService, $LineItemPayment, $hResponse);
        }
    }
}

sub _retrieveServletValues {
    my $self = shift;
    my ($Servlet) = @_;

    my $hQueryParams = $Servlet->params();
    my $BasketOrderId = $hQueryParams->{'OrderId'};
    my $LineItemPaymentGUID = $hQueryParams->{'LineItemPaymentGUID'};
    my $Shop = $Servlet->object();

    return {
        'BasketOrderId'       => $BasketOrderId,
        'LineItemPaymentGUID' => $LineItemPaymentGUID,
        'Shop'                => $Shop
    };
}

sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_SAFERPAY2, $Name, $DebugData, $Section, $Filter);
}

sub _matchLineItemPaymentInShop {
    my $self = shift;
    my ($Shop, $LineItemPayment) = @_;

    return 0 unless ($LineItemPayment->getSite()->id() == $Shop->id());
    return 1;
}

sub _getBasketOrderFromLineItem {
    my $self = shift;
    my ($LineItemPayment) = @_;

    return $LineItemPayment->parent()->parent();
}

1;